// Clause Where

// Opciones para el QUERY

// Crea los querys
class QueryBuilder {
  _columns = "*";
  _joins = [];
  _orderBy = [];
  constructor(db, tableName) {
    this.db = db;
    this.tableName = tableName;
  }
  select(columns) {
    this._columns = columns;
    return this;
  }
  join(table, on, type = "INNER") {
    this._joins.push({
      table,
      on,
      type
    });
    return this;
  }
  where(clause, args) {
    this._where = {
      clause,
      args
    };
    return this;
  }
  groupBy(group) {
    this._groupBy = group;
    return this;
  }
  having(clause, args) {
    this._having = {
      clause,
      args
    };
    return this;
  }
  orderBy(column, direction) {
    this._orderBy.push({
      column,
      direction: direction === null || direction === void 0 ? void 0 : direction.toUpperCase()
    });
    return this;
  }
  limit(limit) {
    this._limit = limit;
    return this;
  }
  page(page) {
    this._page = page;
    return this;
  }
  cursorFactory(factory) {
    this._cursorFactory = factory;
    return this;
  }

  /** 
   * SELECT
   * 
   * @returns array de registros
   */
  async get(cursorFactory) {
    var _this$_where, _this$_having;
    const sql = QueryBuilder.buildSelect(this.tableName, {
      columns: this._columns,
      joins: this._joins,
      where: this._where,
      groupBy: this._groupBy,
      having: this._having,
      order: this._orderBy,
      limit: this._limit,
      page: this._page
    });
    const args = [...(((_this$_where = this._where) === null || _this$_where === void 0 ? void 0 : _this$_where.args) || []), ...(((_this$_having = this._having) === null || _this$_having === void 0 ? void 0 : _this$_having.args) || [])];
    const result = await this.db.rawQuery(sql, args);
    const factory = cursorFactory ?? this._cursorFactory ?? (row => row);
    return Array.from({
      length: result.rows.length
    }, (_, i) => factory(result.rows.item(i)));
  }

  /** 
   * INSERT
   * 
   * @param {any} record { "Column 1": "foo", "Column 2": "bar" }
   * @returns insertId
   */
  async insert(record) {
    var sql = QueryBuilder.buildInsert(this.tableName, record);
    const args = Object.values(record);
    const result = await this.db.executeSql(sql, args);
    return result.insertId ?? -1;
  }

  /** 
   * INSERT
   * 
   * @param {string[]} fields ["Column 1", "Column 2"]
   * @param {any[][]} data [
   *  ["foo", "bar"],
   *  ["abc", "def"]
   * ]
   * @returns result
   */
  insertArray(fields, data) {
    const sql = QueryBuilder.buildInsertArray(this.tableName, fields, data);
    // crea una nueva matriz con todos los elementos de las submatrices 
    // concatenados recursivamente hasta la profundidad especificada
    const args = data.flat();
    return this.db.executeSql(sql, args);
  }

  /** 
   * UPDATE
   * 
   * @param {any} record { "Column 1": "foo", "Column 2": "bar" }
   * @returns rowsAffected
   */
  async update(record) {
    var _this$_where2, _this$_where3;
    const sql = QueryBuilder.buildUpdate(this.tableName, record, (_this$_where2 = this._where) === null || _this$_where2 === void 0 ? void 0 : _this$_where2.clause);
    const args = [...Object.values(record), ...(((_this$_where3 = this._where) === null || _this$_where3 === void 0 ? void 0 : _this$_where3.args) || [])];
    const result = await this.db.executeSql(sql, args);
    return result.rowsAffected;
  }

  /** 
   * DELETE
   * 
   * @returns rowsAffected
   */
  async delete() {
    var _this$_where4, _this$_where5;
    const sql = QueryBuilder.buildDelete(this.tableName, (_this$_where4 = this._where) === null || _this$_where4 === void 0 ? void 0 : _this$_where4.clause);
    const result = await this.db.executeSql(sql, (_this$_where5 = this._where) === null || _this$_where5 === void 0 ? void 0 : _this$_where5.args);
    return result.rowsAffected;
  }

  /**  
   * Creates the "SELECT" sql statement
   * 
   * @param {string} tableName
   * @param {QueryOptions} options
   * @returns string
   */
  static buildSelect(table, opts) {
    const {
      distinct,
      columns = "*",
      joins = [],
      where,
      groupBy,
      having,
      order,
      limit,
      page
    } = opts;
    const sql = ["SELECT", distinct ? "DISTINCT" : "", Array.isArray(columns) ? columns.join(", ") : columns, "FROM", table, ...joins.map(({
      table,
      on,
      type = "INNER"
    }) => `${type} JOIN ${table} ON ${on}`), where !== null && where !== void 0 && where.clause ? `WHERE ${where.clause}` : "", groupBy ? `GROUP BY ${Array.isArray(groupBy) ? groupBy.join(", ") : groupBy}` : "", having !== null && having !== void 0 && having.clause ? `HAVING ${having.clause}` : "", order !== null && order !== void 0 && order.length ? `ORDER BY ${order.map(({
      column,
      direction
    }) => `${column} ${direction ?? ""}`.trim()).join(", ")}` : "", limit ? `LIMIT ${limit}` : "", page && limit ? `OFFSET ${limit * (page - 1)}` : ""];
    return sql.filter(Boolean).join(" ");
  }

  /** 
   * Creates the "INSERT" sql statement
   * 
   * @param {string} tableName
   * @param {any} record { "Column 1": "foo", "Column 2": "bar" }
   * @returns string
   */
  static buildInsert(tableName, record) {
    const keys = Object.keys(record);
    const columns = keys.join(', ');
    const values = keys.map(() => '?').join(', ');
    return `INSERT INTO ${tableName} (${columns}) VALUES (${values});`;
  }

  /** 
   * Creates the "INSERT" sql statement
   * 
   * @param {string} tableName
   * @param {string[]} fields ["Column 1", "Column 2"]
   * @param {any[][]} data [
   *  ["foo", "bar"],
   *  ["abc", "def"]
   * ]
   * @returns string
   */
  static buildInsertArray(tableName, keys, data) {
    if (keys.length === 0) {
      throw new Error('keys is empty or undefined IN INSERT ARRAY');
    }
    if (data.length === 0) {
      throw new Error('data is empty or undefined IN INSERT ARRAY');
    }
    const columns = keys.join(', ');
    const values = keys.map(() => '?').join(', ');
    const valuesArray = data.map(() => `(${values})`).join(', ');
    return `INSERT INTO ${tableName} (${columns}) VALUES ${valuesArray};`;
  }

  /** 
   * Creates the "Update" sql statement
   * 
   * @param {string} tableName
   * @param {any} record { "Column 1": "foo", "Column 2": "bar" }
   * @param {string} whereClause
   * @returns string
   */
  static buildUpdate(tableName, record, whereClause) {
    if (!record || Object.keys(record).length === 0) {
      throw new Error("record is empty or undefined IN UPDATE");
    }

    // Extrae el valor de "id" y crea un nuevo objeto sin ese dato
    //const { id, ...props } = object
    const values = Object.keys(record).map(k => `${k} = ?`).join(', ');
    const wherePart = whereClause && whereClause.trim().length > 0 ? ` WHERE ${whereClause}` : '';
    return `UPDATE ${tableName} SET ${values}${wherePart};`;
  }

  /**
   * Creates the "DELETE" sql statement
   * 
   * @param {string} tableName
   * @param {string} whereClause
   * @returns string
   */
  static buildDelete(tableName, whereClause) {
    const wherePart = whereClause && whereClause.trim().length > 0 ? ` WHERE ${whereClause}` : '';
    return `DELETE FROM ${tableName}${wherePart};`;
  }

  /**
   * Compila una clausula where
   * 
   * @param {any[]} array [1,3,4,2]
   * @returns (?, ?, ?, ?)
   */
  static whereClausePlaceholders(array) {
    return `(${Array(array.length).fill('?').join(', ')})`;
  }

  /**
   * @param sqlString "valor con 'comillas'"
   * @returns "'valor con \'comillas\''"
   */
  static escapeSqlString(sqlString, escape = '\'') {
    const escapedString = sqlString.replace(new RegExp(escape, 'g'), escape + escape);
    return escape + escapedString + escape;
  }
}
export default QueryBuilder;
//# sourceMappingURL=QueryBuilder.js.map