import DB, { type QueryResult } from "./DB";
export type WhereClause = {
    clause?: string;
    args?: any[];
};
export type JoinClause = {
    table: string;
    on: string;
    type?: "INNER" | "LEFT" | "RIGHT";
};
export type OrderBy = {
    column: string;
    direction?: "ASC" | "DESC";
};
export type QueryOptions = {
    distinct?: boolean;
    columns?: string | string[];
    joins?: JoinClause[];
    where?: WhereClause;
    groupBy?: string | string[];
    having?: WhereClause;
    order?: OrderBy[];
    page?: number;
    limit?: number;
};
declare class QueryBuilder {
    db: DB;
    tableName: string;
    private _columns;
    private _joins;
    private _where?;
    private _groupBy?;
    private _having?;
    private _orderBy;
    private _limit?;
    private _page?;
    private _cursorFactory?;
    constructor(db: DB, tableName: string);
    select(columns: string | string[]): this;
    join(table: string, on: string, type?: "INNER" | "LEFT" | "RIGHT"): this;
    where(clause: string, args?: any[]): this;
    groupBy(group: string | string[]): this;
    having(clause: string, args?: any[]): this;
    orderBy(column: string, direction?: "ASC" | "DESC"): this;
    limit(limit: number): this;
    page(page: number): this;
    cursorFactory(factory: (row: any) => any): this;
    /**
     * SELECT
     *
     * @returns array de registros
     */
    get<V>(cursorFactory?: (row: any) => V): Promise<V[]>;
    /**
     * INSERT
     *
     * @param {any} record { "Column 1": "foo", "Column 2": "bar" }
     * @returns insertId
     */
    insert(record: any): Promise<number>;
    /**
     * INSERT
     *
     * @param {string[]} fields ["Column 1", "Column 2"]
     * @param {any[][]} data [
     *  ["foo", "bar"],
     *  ["abc", "def"]
     * ]
     * @returns result
     */
    insertArray(fields: string[], data: any[][]): Promise<QueryResult>;
    /**
     * UPDATE
     *
     * @param {any} record { "Column 1": "foo", "Column 2": "bar" }
     * @returns rowsAffected
     */
    update(record: any): Promise<number>;
    /**
     * DELETE
     *
     * @returns rowsAffected
     */
    delete(): Promise<number>;
    /**
     * Creates the "SELECT" sql statement
     *
     * @param {string} tableName
     * @param {QueryOptions} options
     * @returns string
     */
    static buildSelect(table: string, opts: QueryOptions): string;
    /**
     * Creates the "INSERT" sql statement
     *
     * @param {string} tableName
     * @param {any} record { "Column 1": "foo", "Column 2": "bar" }
     * @returns string
     */
    static buildInsert(tableName: string, record: any): string;
    /**
     * Creates the "INSERT" sql statement
     *
     * @param {string} tableName
     * @param {string[]} fields ["Column 1", "Column 2"]
     * @param {any[][]} data [
     *  ["foo", "bar"],
     *  ["abc", "def"]
     * ]
     * @returns string
     */
    static buildInsertArray(tableName: string, keys: string[], data: any[][]): string;
    /**
     * Creates the "Update" sql statement
     *
     * @param {string} tableName
     * @param {any} record { "Column 1": "foo", "Column 2": "bar" }
     * @param {string} whereClause
     * @returns string
     */
    static buildUpdate(tableName: string, record: any, whereClause?: string): string;
    /**
     * Creates the "DELETE" sql statement
     *
     * @param {string} tableName
     * @param {string} whereClause
     * @returns string
     */
    static buildDelete(tableName: string, whereClause?: string): string;
    /**
     * Compila una clausula where
     *
     * @param {any[]} array [1,3,4,2]
     * @returns (?, ?, ?, ?)
     */
    static whereClausePlaceholders(array: any[]): string;
    /**
     * @param sqlString "valor con 'comillas'"
     * @returns "'valor con \'comillas\''"
     */
    static escapeSqlString(sqlString: string, escape?: string): string;
}
export default QueryBuilder;
//# sourceMappingURL=QueryBuilder.d.ts.map